--[[
  Auctioneer Advanced - Search UI - Searcher Pawn
  Version: 1.0.2 (Xit)
  Revision: $Id: SearcherPawn.lua 1.0.2 2009-03-15 23:45:20 Xit $
  URL: http://wow.curse.com/downloads/wow-addons/details/auc-advanced-searcher-pawn.aspx

  This is a plugin module for the SearchUI that assists in searching by evaluating items with Pawn

  License:
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program(see GPL.txt); if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Note:
    This AddOn's source code is specifically designed to work with
    World of Warcraft's interpreted AddOn system.
    You have an implicit license to use this AddOn with these facilities
    since that is its designated purpose as per:
    http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]
-- Create a new instance of our lib with our parent
local lib, parent, private = AucSearchUI.NewSearcher("Pawn")
if not lib then return end
local print,decode,_,_,replicate,empty,_,_,_,debugPrint,fill = AucAdvanced.GetModuleLocals()
local get,set,default,Const = AucSearchUI.GetSearchLocals()

lib.tabname = "Pawn"

-- Set our defaults
default("search.pawn.scalename", "")
default("search.pawn.enchant", false)
default("search.pawn.canuse", false)
default("search.pawn.forcemh", false)
private.scaletable = {}
private.pclass = UnitClass("player")
private.plevel = UnitLevel("player")

local function getScales()
  local scalecopy = {}
  private.scaletable = PawnGetAllScales()
  table.insert(scalecopy, { 1, "" })
  for i,j in pairs(private.scaletable) do
    table.insert(scalecopy, { i+1, j })
  end
  return scalecopy
end

-- This function is automatically called when we need to create our search generals
function lib:MakeGuiConfig(gui)
  -- Get our tab and populate it with our controls
  local id = gui:AddTab(lib.tabname, "Searchers")

  -- Add the help
  gui:AddSearcher("Pawn", "Search for items which Pawn considers an upgrade", 100)
  gui:AddHelp(id, "pawn searcher",
    "What does this searcher do?",
    "This searcher allows you to choose a scale that has been defined in the Pawn addon.  This searcher will then use Pawn to calculate values for items in the auction scan and only show items which are an upgrade for you.")

  gui:AddControl(id, "Header", 0, "Pawn Search Criteria")
  local last = gui:GetLast(id)
  gui:AddControl(id, "Selectbox",  0, 1, getScales(), "search.pawn.scalename", "ScaleName")
  local cont = gui:GetLast(id)
  gui:AddTip(id, "The Pawn Scale that will be used to determine the item value")
  gui:SetLast(id, last)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.enchant", "Also Check Enchant Value")
  gui:SetLast(id, last)
  gui:AddControl(id, "Checkbox",   0.55, 0, "search.pawn.canuse", "Useable Only")
  gui:SetLast(id, cont)
  last = cont
  gui:AddControl(id, "Checkbox",   0.24, 1, "search.pawn.forcemh", "Force Main Hand")
  gui:AddTip(id, "Force the search to consider one-hand weapons same as main hand only.")
end

local function Debug(Text)
  if DEFAULT_CHAT_FRAME then
    DEFAULT_CHAT_FRAME:AddMessage(tostring(Text))
  else
    message(tostring(Text))
  end
end

local function RGBPercToHex(r, g, b)
  r = r <= 1 and r >= 0 and r or 0
  g = g <= 1 and g >= 0 and g or 0
  b = b <= 1 and b >= 0 and b or 0
  return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

local function CanUse(link)

  MyScanningTooltip:ClearLines()
  MyScanningTooltip:SetHyperlink(link)
  local retval = true

  for i=1,MyScanningTooltip:NumLines() do
    --Check left side text to see if it is red, if so, can't use
    local mytext = getglobal("MyScanningTooltipTextLeft" .. i)
    local text = mytext:GetText()
    if (text) then
      local r,g,b,a = mytext:GetTextColor()
      local hex = RGBPercToHex(r,g,b)
      if (hex == "fe1f1f") then
        retval = false
      end
    end

    --Check right side text to see if it is red, if so, can't use
    local mytextr = getglobal("MyScanningTooltipTextRight" .. i)
    local textr = mytextr:GetText()
    if (textr) then
      local r,g,b,a = mytextr:GetTextColor()
      local hex = RGBPercToHex(r,g,b)
      if (hex == "fe1f1f") then
        retval = false
      end
    end
  end

  return retval
end

-- Convert between auctioneer equip positions and Pawn equip positions
local function ConvertSlot(itemData)
  local mainslot, otherslot

  local pos = itemData[Const.IEQUIP]
  local itype = itemData[Const.ITYPE]
  local subtype = itemData[Const.ISUB]

  local candual = 0
  local hasammo = 0
  local forcemh = get("search.pawn.forcemh")

  -- Limit one-handed weapon checks to main hand unless class can dual wield
  if (private.pclass == "Warrior" or private.pclass == "Hunter" and private.plevel >= 20) then candual = 1 end
  if (private.pclass == "Shaman" and private.plevel >= 40) then candual = 1 end
  if (private.pclass == "Rogue" and private.plevel >= 10) then candual = 1 end
  if (private.pclass == "Death Knight") then candual = 1 end

  if pos == 11 then  -- Finger
    mainslot = 11
    otherslot = 12
  elseif pos == 12 then -- Trinket
    mainslot = 13
    otherslot = 14
  elseif pos == 13 then -- One Hand Weapon
    mainslot = 16
    if candual and not forcemh then otherslot = 17 end
  elseif pos == 14 then -- Shield
    mainslot = 17
  elseif pos == 15 then -- Ranged Right
    mainslot = 18
  elseif pos == 16 then -- Cloak
    mainslot = 15
  elseif pos == 17 then -- 2Handed
    mainslot = 16
  elseif pos == 20 then -- Robe
    mainslot = 5
  elseif pos == 21 then -- Main Hand Weapon
    mainslot = 16
  elseif pos == 22 then -- Off hand
    mainslot = 17
  elseif pos == 23 then -- Holdable
    mainslot = 17
  elseif pos == 24 then -- Ammo
    mainslot = 0
  elseif pos == 25 then -- Thrown
    mainslot = 18
  elseif pos == 26 then -- Ranged
    mainslot = 18
  elseif (itype == "Armor") and -- Relics
     ((subtype == "Totems") or (subtype == "Librams") or (subtype == "Idols") or (subtype == "Sigils"))
  then
    mainslot = 18
  else
    mainslot = pos 
  end

  return mainslot, otherslot
end

function lib.Search(itemData)
  link = itemData[Const.LINK]
  itype = itemData[Const.ITYPE]

  if not link then return false, "No link" end
  
  -- Don't show Ammo/thrown for non ammo classes
  if ((private.pclass ~= "Hunter" and private.pclass ~= "Rogue" and private.pclass ~= "Warrior") and
      (itemData[Const.IEQUIP] == 24 or itemData[Const.IEQUIP] == 25)) then return false, "Can't Use" end

  -- Only filter stuff that can be equipped
  if (not IsEquippableItem(link)) then return false, "Not Equippable" end

  local canuse = get("search.pawn.canuse")

  -- Check to see if we can use the item
  local icanuse = CanUse(link)

  if canuse and icanuse ~= true then return false, "Can't use" end

  -- Get the signature of this item and find it's stats.
  local item = PawnGetItemData(link)

  if not item then return false, "No Item" end

  -- Does the stored filter exist
  local scalenum = get("search.pawn.scalename")
  local scalename = ""

  if not scalenum then return false, "No Scale Name" end

  for i, j in pairs(private.scaletable) do
    if i == (scalenum - 1) then -- subtract 1, due to offset introduced in menu
      scalename = j
      break
    end
  end

  if scalename == "" then return false, "Scale not found or blank" end

  local scalefound = PawnDoesScaleExist(scalename)
  if not scalefound or scalefound ~= true then return false, "Scale not found" end

  local aVal, aValE = PawnGetSingleValueFromItem(item, scalename)

  -- Get value of equivalent slot(s).
  local mainslot, otherslot = ConvertSlot(itemData)
  local mItem = PawnGetItemDataForInventorySlot(mainslot)
  local mVal, mValE = PawnGetSingleValueFromItem(mItem, scalename)
  local oItem = nil
  local oVal = 0
  local oValE = 0

  if (otherslot) then
    oItem = PawnGetItemDataForInventorySlot(otherslot)
    oVal, oValE = PawnGetSingleValueFromItem(oItem, scalename)
  end

  -- ensure we have valid values for comparisons

  if aVal == nil then aVal = 0 end
  if aValE == nil then aValE = 0 end
  if mVal == nil then mVal = 0 end
  if mValE == nil then mValE = 0 end
  if oVal == nil then oVal = 0 end
  if oValE == nil then oValE = 0 end

  local aStr = ("%0.2f"):format(aVal);
  local aStrE = ("(E)%0.2f"):format(aValE);

  -- compare 2-handed weapons against both main hand and off hand
  if itemData[Const.IEQUIP] == 17 then
    local bVal = mVal + oVal
    if aVal > bVal then return aStr,nil,nil end
  end

  if aVal > mVal then 
  return aStr,nil,nil end
  if otherslot and aVal > oVal then
  return aStr,nil,nil end

  local enchant = get("search.pawn.enchant")
  if (enchant) then
    -- compare 2-handed weapons against both main hand and off hand
    if itemData[Const.IEQUIP] == 17 then
      local bValE = mValE + oValE
      if aValE > bValE then return aStrE,nil,nil end
    end
    if aValE > mValE then
    return aStrE,nil,nil end
    if otherslot and aValE > oValE then
    return aStrE,nil,nil end
  end

  return false, "Pawn Value "..aVal.." too low"
end
